@@ -112,6 +112,7 @@ class SOCol(object):
                  creationOrder,
                  dbName=None,
                  default=NoDefault,
+                 defaultSQL=None,
                  foreignKey=None,
                  alternateID=False,
                  alternateMethodName=None,
@@ -178,6 +179,7 @@ class SOCol(object):
         self.name = name
         self.soClass = soClass
         self._default = default
+        self.defaultSQL = defaultSQL
         self.customSQLType = sqlType
 
         # deal with foreign keys
@@ -285,7 +287,7 @@ class SOCol(object):
         return r + '>'
 
     def createSQL(self):
-        return ' '.join([self._sqlType() + self._extraSQL()])
+        return ' '.join([self._sqlType()] + self._extraSQL())
 
     def _extraSQL(self):
         result = []
@@ -293,6 +295,8 @@ class SOCol(object):
             result.append('NOT NULL')
         if self.unique or self.alternateID:
             result.append('UNIQUE')
+        if self.defaultSQL is not None:
+            result.append("DEFAULT %s" % self.defaultSQL)
         return result
 
     def _sqlType(self):
@@ -594,6 +598,11 @@ class IntValidator(validators.Validator):
 
 class SOIntCol(SOCol):
     # 3-03 @@: support precision, maybe max and min directly
+    def __init__(self, **kw):
+        self.length = popKey(kw, 'length')
+        self.unsigned = bool(popKey(kw, 'unsigned'))
+        self.zerofill = bool(popKey(kw, 'unsigned'))
+        SOCol.__init__(self, **kw)
 
     def autoConstraints(self):
         return [consts.isInt]
@@ -602,12 +611,54 @@ class SOIntCol(SOCol):
         return [IntValidator(name=self.name)] + \
             super(SOIntCol, self).createValidators()
 
+    def addSQLAttrs(self, str):
+        _ret = str
+        if str is None or len(str) < 1:
+            return None
+
+        if self.length >= 1:
+            _ret = "%s(%d)" % (_ret, self.length)
+        if self.unsigned:
+            _ret = _ret + " UNSIGNED"
+        if self.zerofill:
+            _ret = _reg + " ZEROFILL"
+        return _ret
+
     def _sqlType(self):
-        return 'INT'
+        return self.addSQLAttrs("INT")
 
 class IntCol(Col):
     baseClass = SOIntCol
 
+class SOTinyIntCol(SOIntCol):
+    def _sqlType(self):
+        return self.addSQLAttrs("TINYINT")
+
+class TinyIntCol(Col):
+    baseClass = SOTinyIntCol
+
+class SOSmallIntCol(SOIntCol):
+    def _sqlType(self):
+        return self.addSQLAttrs("SMALLINT")
+
+class SmallIntCol(Col):
+    baseClass = SOSmallIntCol
+
+class SOMediumIntCol(SOIntCol):
+    def _sqlType(self):
+        return self.addSQLAttrs("MEDIUMINT")
+
+class MediumIntCol(Col):
+    baseClass = SOMediumIntCol
+
+class SOBigIntCol(SOIntCol):
+    def _sqlType(self):
+        return self.addSQLAttrs("BIGINT")
+
+class BigIntCol(Col):
+    baseClass = SOBigIntCol
+
+
 class BoolValidator(validators.Validator):
 
     def to_python(self, value, state):
@@ -879,7 +930,8 @@ class SOEnumCol(SOCol):
         return [consts.isString, consts.InList(self.enumValues)]
 
     def createValidators(self):
-        return [EnumValidator(name = self.name, enumValues = self.enumValues)] + \
+        return [EnumValidator(name = self.name, enumValues = self.enumValues,
+                              notNone = self.notNone)] + \
             super(SOEnumCol, self).createValidators()
 
     def _mysqlType(self):
@@ -929,6 +981,8 @@ class EnumValidator(validators.Validator):
     def to_python(self, value, state):
         if value in self.enumValues:
             return value
+        elif not self.notNone and value is None:
+            return None
         else:
             raise validators.Invalid("expected a member of %r in the EnumCol '%s', got %r instead" % \
                                      (self.enumValues, self.name, value), value, state)
@@ -940,6 +994,43 @@ class EnumCol(Col):
     baseClass = SOEnumCol
 
 
+class SetValidator(validators.Validator):
+    """
+    Translates Python tuples into SQL comma-delimited SET strings.
+    """
+
+    def to_python(self, value, state):
+        if not isinstance(value, str):
+            raise validators.Invalid("expected a value string, got % instead" % \
+                                      (value), value, state)
+        return tuple(value.split(","))
+
+    def from_python(self, value, state):
+        if not isinstance(value, tuple):
+            value = (value,)
+        return ",".join([v for v in value])
+
+class SOSetCol(SOCol):
+    def __init__(self, **kw):
+        self.setValues = popKey(kw, 'setValues', None)
+        assert self.setValues is not None, \
+                'You must provide a setValues keyword argument'
+        super(SOSetCol, self).__init__(**kw)
+
+    def autoConstraints(self):
+        return [consts.isString, consts.InList(self.setValues)]
+
+    def createValidators(self):
+        return [SetValidator(name = self.name, setValues = self.setValues)] + \
+            super(SOSetCol, self).createValidators()
+
+    def _mysqlType(self):
+        return "SET(%s)" % ', '.join([sqlbuilder.sqlrepr(v, 'mysql') for v in self.enumValues])
+
+class SetCol(Col):
+    baseClass = SOSetCol
+
+
 if datetime_available:
     class DateTimeValidator(validators.DateValidator):
         def to_python(self, value, state):
@@ -1063,7 +1154,6 @@ class DateTimeCol(Col):
                        % DATETIME_IMPLEMENTATION)
     now = staticmethod(now)
 
-
 if datetime_available:
     class DateValidator(DateTimeValidator):
         def to_python(self, value, state):
@@ -1182,6 +1272,21 @@ class TimeCol(Col):
     baseClass = SOTimeCol
 
 
+class SOTimestampCol(SODateTimeCol):
+    """
+    Necessary to support MySQL's use of TIMESTAMP versus DATETIME types
+    """
+
+    def __init__(self, **kw):
+        SOCol.__init__(self, **kw)
+
+    def _mysqlType(self):
+        return 'TIMESTAMP'
+
+class TimestampCol(Col):
+    baseClass = SOTimestampCol
+
+
 try:
     from decimal import Decimal
 except ImportError:
@@ -1357,6 +1462,7 @@ class SOPickleCol(SOBLOBCol):
 class PickleCol(BLOBCol):
     baseClass = SOPickleCol
 
+
 def popKey(kw, name, default=None):
     if not kw.has_key(name):
         return default
